import React, { useEffect, useState } from 'react';
import { Add, Delete, Settings } from '@mui/icons-material';
import { useAppDispatch, useSchemaGraph, useSessionCache, useAuthorizationCache } from '@graphpolaris/shared/lib/data-access';
import { deleteSaveState, selectSaveState } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { SettingsForm } from './forms/settings';
import { LoadingSpinner } from '@graphpolaris/shared/lib/components/LoadingSpinner';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { DropdownButton, DropdownContainer, DropdownItemContainer } from '@graphpolaris/shared/lib/components/dropdowns';
import { clearQB } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { clearSchema } from '@graphpolaris/shared/lib/data-access/store/schemaSlice';
import { DatabaseStatus, SaveStateI, wsDeleteState } from '@graphpolaris/shared/lib/data-access/api/wsState';

export default function DatabaseSelector({}) {
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const schemaGraph = useSchemaGraph();
  const authCache = useAuthorizationCache();
  const dbSelectionMenuRef = React.useRef<HTMLDivElement>(null);
  const [hovered, setHovered] = useState<string | null>(null);
  const [connecting, setConnecting] = useState<boolean>(false);
  const [dbSelectionMenuOpen, setDbSelectionMenuOpen] = useState<boolean>(false);
  const [settingsMenuOpen, setSettingsMenuOpen] = useState<'create' | 'update' | undefined>(undefined);
  const [selectedSaveState, setSelectedSaveState] = useState<SaveStateI | null>(null);
  // const [addDbConnectionFormOpen, setAddDbConnectionFormOpen] = useState<boolean>(false);

  useEffect(() => {
    const handleClickOutside = ({ target }: MouseEvent) => {
      if (dbSelectionMenuRef.current && !dbSelectionMenuRef.current.contains(target as Node)) {
        setDbSelectionMenuOpen(false);
      }
    };
    document.addEventListener('click', handleClickOutside);
    return () => {
      document.removeEventListener('click', handleClickOutside);
    };
  }, []);

  useEffect(() => {
    setConnecting(false);
  }, [schemaGraph]);

  useEffect(() => {
    let timeoutId: ReturnType<typeof setTimeout>;
    if (connecting) {
      timeoutId = setTimeout(() => {
        dispatch(addError("Couldn't establish connection"));
        setConnecting(false);
        dispatch(selectSaveState(undefined));
        dispatch(clearQB());
        dispatch(clearSchema());
      }, 10000);
    }

    return () => {
      if (timeoutId) clearTimeout(timeoutId);
    };
  }, [connecting]);

  return (
    <>
      {settingsMenuOpen !== undefined && (
        <SettingsForm
          open={settingsMenuOpen}
          saveState={settingsMenuOpen === 'update' ? selectedSaveState : null}
          onClose={() => {
            setSettingsMenuOpen(undefined);
          }}
        />
      )}
      <DropdownContainer ref={dbSelectionMenuRef} className="w-[20rem]">
        <DropdownButton
          disabled={connecting || authCache.authorized === false || !!authCache.roomID}
          title={
            <div className="flex items-center">
              {connecting && session.currentSaveState && session.currentSaveState in session.saveStates ? (
                <>
                  <LoadingSpinner />
                  <p className="ml-2 truncate">Connecting to {session.saveStates[session.currentSaveState].name}</p>
                </>
              ) : session.currentSaveState && session.currentSaveState in session.saveStates ? (
                <>
                  <div
                    className={`h-2 w-2 rounded-full ${
                      session.testedSaveState[session.currentSaveState] === DatabaseStatus.tested ? 'bg-success-500' : 'bg-danger-500'
                    }`}
                  />
                  <p className="ml-2 truncate">Connected DB: {session.saveStates[session.currentSaveState].name}</p>
                </>
              ) : session.saveStates === undefined ? (
                <>
                  <LoadingSpinner />
                  <p className="ml-2">Retrieving databases</p>
                </>
              ) : Object.keys(session.saveStates).length === 0 ? (
                <>
                  <p className="ml-2">Add your first Database</p>
                </>
              ) : (
                <>
                  <div className="h-2 w-2 rounded-full bg-secondary-500" />
                  <p className="ml-2">Select a database</p>
                </>
              )}
            </div>
          }
          onClick={() => {
            if (session.saveStates && Object.keys(session.saveStates).length === 0) setSettingsMenuOpen('create');
            else setDbSelectionMenuOpen(!dbSelectionMenuOpen);
          }}
        />

        {dbSelectionMenuOpen && session.saveStates !== undefined && (
          <DropdownItemContainer align="top-10 w-full">
            <li
              className="flex items-center p-2 hover:bg-secondary-50 cursor-pointer"
              onClick={(e) => {
                e.preventDefault();
                setDbSelectionMenuOpen(false);
                setConnecting(false);
                setSettingsMenuOpen('create');
              }}
              title="Add new database"
            >
              {session.saveStates && Object.keys(session.saveStates).length === 0 ? (
                <>
                  <Add />
                  <p className="ml-2">Add your first database</p>
                </>
              ) : (
                <>
                  <Add />
                  <p className="ml-2">Add database</p>
                </>
              )}
            </li>
            {Object.values(session.saveStates).map((save) => (
              <li
                key={save.id}
                className="flex justify-between items-center px-4 py-2 hover:bg-primary-100 gap-2 cursor-pointer"
                onClick={(e) => {
                  if (save.id !== session.currentSaveState) {
                    e.preventDefault();
                    setDbSelectionMenuOpen(false);
                    setConnecting(true);
                    dispatch(selectSaveState(save.id));
                    dispatch(clearQB());
                    dispatch(clearSchema());
                  } else {
                    setDbSelectionMenuOpen(false);
                  }
                }}
                onMouseEnter={() => setHovered(save.id)}
                onMouseLeave={() => setHovered(null)}
                title={`Connect to ${save.name}`}
              >
                <div
                  className={`h-[8px] w-[8px] rounded-full shrink-0 ${
                    session.testedSaveState[save.id] === DatabaseStatus.tested ? 'bg-success-500' : 'bg-danger-500'
                  }`}
                />
                <div className="w-full shrink min-w-0 flex flex-col">
                  <p className="truncate w-full shrink-0 min-w-0">{save.name}</p>
                  <p className="bg-light text-2xs text-secondary-500 truncate w-fit shrink-0 min-w-0 max-w-full h-full border border-secondary-200 rounded-sm p-0.5">
                    {save.db.protocol}
                    {save.db.url}
                  </p>
                </div>
                {hovered === save.id && (
                  <div className="flex items-center ml-2">
                    <div
                      className="text-secondary-700 hover:text-secondary-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        e.stopPropagation();
                        setSettingsMenuOpen('update');
                        setSelectedSaveState(save);
                      }}
                    >
                      <Settings />
                    </div>
                    <div
                      className="text-secondary-700 hover:text-secondary-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        e.stopPropagation();
                        if (session.currentSaveState === save.id) {
                          dispatch(clearQB());
                          dispatch(clearSchema());
                        }
                        wsDeleteState(save.id);
                        dispatch(deleteSaveState(save.id));
                      }}
                      title="Delete database connection"
                    >
                      <Delete />
                    </div>
                  </div>
                )}
              </li>
            ))}
          </DropdownItemContainer>
        )}
      </DropdownContainer>
    </>
  );
}
