import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import {
  databaseNameMapping,
  databaseProtocolMapping,
  useAppDispatch,
  SaveStateI,
  wsUpdateState,
  DatabaseStatus,
  wsTestDatabaseConnection,
  TestDatabaseConnectionResponse,
  wsCreateState,
  nilUUID,
  DatabaseType,
  useAuthorizationCache,
} from '@graphpolaris/shared/lib/data-access';
import { ErrorOutline } from '@mui/icons-material';
import { Dialog } from '@graphpolaris/shared/lib/components/Dialog';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import { useImmer } from 'use-immer';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import { addSaveState, testedSaveState } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { DatabaseForm, INITIAL_SAVE_STATE } from './databaseForm';
import { SampleDatabaseSelector } from './mockSaveStates';

type Connection = {
  updating: boolean;
  status: null | string;
  verified: boolean | null;
};

export const SettingsForm = (props: { onClose(): void; open: 'create' | 'update'; saveState: SaveStateI | null }) => {
  const dispatch = useAppDispatch();
  const ref = useRef<HTMLDialogElement>(null);
  const auth = useAuthorizationCache();
  const [formData, setFormData] = useImmer(
    props.saveState && props.open === 'update' ? props.saveState : { ...INITIAL_SAVE_STATE, user_id: auth.userID || '' },
  );
  const [hasError, setHasError] = useState(false);
  const [sampleDataPanel, setSampleDataPanel] = useState<boolean | null>(false);
  const [connection, setConnection] = useState<Connection>({
    updating: false,
    status: null,
    verified: null,
  });
  const formDataRef = useRef<SaveStateI | null>(null);
  const formTitle = props.open === 'create' ? 'Create' : 'Update';

  const refImperativeHandles = useRef<any>(null);
  useImperativeHandle(refImperativeHandles, () => ({
    processDbTested: (data: TestDatabaseConnectionResponse) => {
      if (!formDataRef.current) {
        console.error('formDataRef.current is null');
        return;
      }
      if (formDataRef.current.user_id !== auth.userID && auth.userID) {
        console.error('formDataRef.current.user_id is not equal to auth.userID');
        formDataRef.current.user_id = auth.userID;
      }
      if (data.status === 'success') {
        setConnection((prevState) => ({
          updating: false,
          status: 'Database connection verified',
          verified: true,
        }));
        if (props.open === 'create') {
          wsCreateState(formDataRef.current);
        } else {
          wsUpdateState(formDataRef.current);
        }
      } else {
        setConnection((prevState) => ({
          updating: false,
          status: 'Database connection failed',
          verified: false,
        }));
      }
    },
    processStateUpdated: (data: SaveStateI) => {
      let _data = JSON.parse(JSON.stringify(data));
      _data.db.status = DatabaseStatus.tested;
      dispatch(addSaveState(_data));
      dispatch(testedSaveState(_data.id));
      formDataRef.current = null;
      closeDialog();
    },
  }));

  useEffect(() => {
    Broker.instance().subscribe(refImperativeHandles.current.processDbTested, 'tested_connection');
    Broker.instance().subscribe(refImperativeHandles.current.processStateUpdated, 'updated_save_state');
    Broker.instance().subscribe(refImperativeHandles.current.processStateUpdated, 'save_state');

    return () => {
      Broker.instance().unSubscribeAll('tested_connection');
      Broker.instance().unSubscribeAll('updated_save_state');
      Broker.instance().unSubscribeAll('save_state');
    };
  }, []);

  useEffect(() => {
    if (props.saveState && props.open === 'update') {
      setFormData(props.saveState);
      setSampleDataPanel(null);
    } else {
      setSampleDataPanel(false);
    }
  }, [props.saveState]);

  useEffect(() => {
    formDataRef.current = formData;
  }, [formData]);

  async function handleSubmit() {
    setConnection(() => ({
      updating: true,
      status: formTitle.slice(0, -1) + 'ing database credentials',
      verified: null,
    }));

    wsTestDatabaseConnection(formData.db);
  }

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port)))
      setFormData((draft) => {
        draft.db.port = Number(port);
        return draft;
      });
  }

  function closeDialog(): void {
    setConnection({
      updating: false,
      status: null,
      verified: null,
    });
    setFormData(INITIAL_SAVE_STATE);
    ref.current?.close();
    props.onClose();
  }

  return (
    <Dialog open={!!props.open} onClose={props.onClose} className="lg:min-w-[50rem]">
      <div className="flex justify-between align-center">
        <h1 className="text-xl font-bold">
          {formTitle} {formData.name} Database
        </h1>
        <div>
          {sampleDataPanel === true ? (
            <Button variant="outline" label="Go back" onClick={() => setSampleDataPanel(false)} />
          ) : sampleDataPanel === false ? (
            <>
              <h1 className="font-light text-xs">No data?</h1>
              <p className="font-light text-sm cursor-pointer underline" onClick={() => setSampleDataPanel(true)}>
                Try sample data
              </p>
            </>
          ) : (
            ''
          )}
        </div>
      </div>

      <>
        {sampleDataPanel === true ? (
          <SampleDatabaseSelector
            onClick={(data) => {
              setFormData(data);
              setHasError(false);
              setSampleDataPanel(false);
            }}
          />
        ) : (
          <DatabaseForm
            data={formData}
            onChange={(data: SaveStateI, error: boolean) => {
              setFormData({ ...data, id: formData.id });
              setHasError(error);
            }}
          />
        )}

        {!(connection.status === null) && (
          <div className={`flex flex-col justify-center items-center`}>
            <div className="flex justify-center items-center">
              {connection.verified === false && <ErrorOutline className="text-secondary-400" />}
              <p className="font-light text-sm	text-secondary-400	">{connection.status}</p>
            </div>
            {connection.verified === null && <progress className="progress w-56"></progress>}
          </div>
        )}

        <div className="grid md:grid-cols-2 gap-3 card-actions w-full justify-stretch items-center">
          <Button
            type="primary"
            label={connection.updating ? formTitle.slice(0, -1) + 'ing...' : formTitle}
            onClick={(event) => {
              event.preventDefault();
              handleSubmit();
            }}
            disabled={connection.updating || hasError}
          />
          <Button
            variant="outline"
            label="Cancel"
            onClick={(event) => {
              event.preventDefault();
              closeDialog();
            }}
          />
        </div>
      </>
    </Dialog>
  );
};
